var Cmp07000_RelacaoFornecedoresAFAnalise = function (ScreenService, General, restEngine, MessageService) {
    /* 
    Devido a passagem de função por parâmetro, ao voltar e ser executada, o 
    contexto do this já não é mais o mesmo. Para contornar e executá-lo, o 
    this é encapsulado num objeto, permanecendo assim a sua referência.
    Equivale à função bind do JavaScript.
    
    "[...]Um objeto ao qual a palavra-chave this pode fazer referência dentro da nova função.[...]"
    - Retirado da documentação da Microsoft sobre a função bind JavaScript;
    */
    var self = this;
    
    /**
     * Define os valores default dos campos.
     * @param {Object} widget - widget principal.
     */
    this.valoresDefault = function(widget) {
        const currentRowWidget = widget.currentRow;
        
        General.callRestEngine("cmp#/getCdFilCentCmp", "Row", {}).then(function(response) {
            setLocalVar('NMFILCENTCMP', response.dataset.data.NMFILCENTCMP);
            setLocalVar('CDFILCENTCMP', response.dataset.data.CDFILCENTCMP);
            
            widget.newRow();
            
            currentRowWidget.REPORT_NAME = 'CMP07003';
            
            currentRowWidget.PERIODO_GERACAO = { 
                'formattedStart': moment().format('L'), 
                'formattedEnd': moment().format('L'), 
                'start': moment().format('L'), 
                'end': moment().format('L') 
            };
            
            currentRowWidget.PERIODO_CONCLUSAO = { 
                'formattedStart': moment().format('L'), 
                'formattedEnd': moment().format('L'), 
                'start': moment().format('L'), 
                'end': moment().format('L') 
            };
            
            currentRowWidget.SITU_AF = ['Baixa Parcial', 'Baixa Manual', 'Encerrada', 'Em Análise', 'Baixa Total', 'Em Aberto', 'Cancelada', 'Aguardando EDI'];
            currentRowWidget.ENVIO_AF = ['Normal', 'EDI', 'Não envia AF'];
            
            // Default dataSourceFilter do campo 'Apuração'
            widget.getField('NRAPURPREC').dataSourceFilter = [
                {
                    'name': 'P_CDCOMPRADOR',
                    'operator': '=',
                    'value': 'T'
                },
                {
                    'name': 'P_CDFILCENTCMP',
                    'operator': '=',
                    'value': getLocalVar('CDFILCENTCMP')
                },
                {
                    'name': 'P_1',
                    'operator': '=',
                    'value': 'Em análise'
                },
                {
                    'name': 'P_2',
                    'operator': '=',
                    'value': 'Concluída'
                },
                {
                    'name': 'P_3',
                    'operator': '=',
                    'value': 'Concluída com envio de e-mail'
                },
                {
                    'name': 'P_4',
                    'operator': '=',
                    'value': 'Concluída com aviso para o fornecedor'
                },
                {
                    'name': 'P_5',
                    'operator': '=',
                    'value': 'Em Análise - OL'
                },
                {
                    'name': 'P_6',
                    'operator': '=',
                    'value': 'Consolidada'
                },
                {
                    'name': 'P_7',
                    'operator': '=',
                    'value': 'Concluída com envio de e-mail - OL'
                },
                {
                    'name': 'P_8',
                    'operator': '=',
                    'value': 'Concluída com aviso para o fornecedor - OL'
                }
            ];
        
            // Pega o parâmetro P_CDGRUPPROD, necessário para o field de Sub-Grupo
            General.callRestEngine('cmp_fornecimento_emissaoAutorizacaoFornecimento#/getCodigoGrupProd', 'Row', {'CDFILCENTCMP': getLocalVar('CDFILCENTCMP')}).then(function(response) {
                widget.getField('NMSUBGRPROD').dataSourceFilter = [
                    {
                        "operator": "ORDER_BY",
                        "name": [
                            {
                                "name": "CDSUBGRPROD",
                                "direction": "ASC"
                            }
                        ]
                    },
                    {
                        "name": "P_CDGRUPPROD",
                        "operator": "=",
                        "value": response.dataset.data[0].CDGRUPPROD
                    },
                    {
                        "name": "NRORG",
                        "operator": "=",
                        "value": getLocalVar('NRORG')
                    }
                ];
            }, function(error) {
                ScreenService.showMessage(error.data ? error.data.error : 'Timeout', 'error');
            });
        }, function(error) {
			return MessageService.showMessageByCode(error.data ? 'cmp#' + error.data.error : 'Timeout', null, { type: 'ERROR' });
		});
    };
    
    /**
     * Retorna a concatenação do array passado pro parâmetro. 
     * @param {Array} arr - array a ser concanetado.
     * @param {string} defaultValue - Valor a ser retornado caso o arr esteja
     * vazio ou em branco.
     * @returns {string} Array concatenado ou o defaultValue passado por
     * parâmetro. Se não houver defaultValue, retorna 'T'.
     */
    this.concatArrayForParams = function (arr, defaultValue) {
        if(!Util.isEmptyOrBlank(arr)) {
            return arr.join('_');
        } else {
            return defaultValue || 'T';
        }
    };
    
    /**
     * Concatena os itens selecionados do campo escolhido. Os campos a que essa
     * função se destina são:
     * - 'Situação da AF';
     * - 'Envio da AF'.
     * Obs.: O item de número 8 é representado como 0.
     * @param {Array} data - campo 'data' do dataSource do campo.
     * @returns {String} Número dos itens concatenados por underline.
     * Ex.: '_1_2_3_4_5_6_7_0'.
     */
    this.formataParamSituAndEnvioAf = function (data) {
        let newIndex,
            retorno = '',
            i = 0;
        
        for (; i < data.length; i++) {
            if (data[i].__isSelected) {
                newIndex = i + 1;
                retorno += '_' + (newIndex === 8 ? 0 : newIndex);
            }
        }
        
        return retorno;
    };
    
    /**
     * Pega o campo 'data' do dataSource do campo escolhido.
     * @params {Object} widget - widget contendo o campo escolhido.
     * @params {Object} field - campo escolhido.
     * @returns {Array} 'data' do campo escolhido.
     */
    this.getDataFromField = function(widget, field) {
        return widget.getField(field).dataSource.data;
    };
    
    this.setDateFieldProperties = function(widget, fieldName) {
        const fieldValidade = widget.getField(fieldName);
        const currentRowWidget = widget.currentRow;
        let startCampoData = '';
        let endCampoData = '';
        
        //Selecionando o campo data de acordo com o fieldName enviado
        if (fieldName === "PERIODO_GERACAO") {
            startCampoData = currentRowWidget.PERIODO_GERACAO.start;
            endCampoData = currentRowWidget.PERIODO_GERACAO.end;
        } else {
            startCampoData = currentRowWidget.PERIODO_CONCLUSAO.start;
            endCampoData = currentRowWidget.PERIODO_CONCLUSAO.end;
        }
        
        //OU Exclusivo - XOR.
        if ((startCampoData && !endCampoData) || (!startCampoData && endCampoData)) {
            fieldValidade.validations = {
                "requiredInterval": true
            };
        } else {
            delete fieldValidade.validations;
            
            if (startCampoData == '' && endCampoData == '' && fieldName === "PERIODO_GERACAO") {
                widget.getField("COINC_PERIODO").checked = false;
            }
        }
        
    };
    
    /**
     * Monta os parâmetros para gerar o relatório.
     * @returns {Object} Parâmetros preparados para o relatório.
     */
    this.montaParametros = function(widget) {
        const getFormattedEndProductCode = code => code ? General.fill(code, 10, true, 'Z') : '';
        const currentRow = widget.currentRow;
        const isBirt = ['CMP07000', 'CMP07002'].includes(currentRow.REPORT_NAME);
        const regionCode = this.concatArrayForParams(currentRow.NMREGIAO);
        const branchOfficeCode = this.concatArrayForParams(currentRow.NMFILIAL);
        const operatorCode = this.concatArrayForParams(currentRow.NMOPERADOR);
        const providerCode = this.concatArrayForParams(currentRow.NMRAZSOCFORN);
        const subgroupList = this.concatArrayForParams(currentRow.NMSUBGRPROD);
        const nrSeqApurAF = this.concatArrayForParams(currentRow.NRAPURPREC);
        const idBaixaAF = this.formataParamSituAndEnvioAf(this.getDataFromField(widget, 'SITU_AF'));
        const sendTypeAF = this.formataParamSituAndEnvioAf(this.getDataFromField(widget, 'ENVIO_AF'));
        const allBranchOfficeGrouping = Util.isEmptyOrBlank(currentRow.NMGRUPFILI) ? 'T' : 'X';
        const endProductCode = getFormattedEndProductCode(currentRow.CDPRODUTOFIN);
        const parameters = {
            'REPORT_NAME': currentRow.REPORT_NAME,
            'QR2TYPE': currentRow.QR2TYPE || 'PDF',
            'CDREGIAO': regionCode,
            'CDREGIAO1': regionCode,
            'CDFILIAL': branchOfficeCode,
            'CDFILIAL1': branchOfficeCode,
            'CDOPERADOR': operatorCode,
            'CDOPERADOR1': operatorCode,
            'CDFORNECED': providerCode,
            'CDFORNECED1': providerCode,
            'LISTASUBGRUP': subgroupList,
            'LISTASUBGRUP1': subgroupList,
            'NRSEQAPURAF': nrSeqApurAF,
            'NRSEQAPURAF1': nrSeqApurAF,
            'COINCIDE': currentRow.COINC_PERIODO,
            'COINCIDE1': currentRow.COINC_PERIODO,
            'IDBAIXAAF': idBaixaAF,
            'IDBAIXAAF1': idBaixaAF,
            'TPENVIOAF': sendTypeAF,
            'TPENVIOAF1': sendTypeAF,
            'PRODUTINI': currentRow.CDPRODUTOINI || '',
            'PRODUTFIN': currentRow.CDPRODUTOFIN || '',
            'PRODINI': currentRow.CDPRODUTOINI || '',
            'PRODINI1': currentRow.CDPRODUTOINI || '',
            'PRODFIN': endProductCode,
            'PRODFIN1': endProductCode,
            'TPRODUTO': currentRow.CDPRODUTOINI ? '' : 'T', 
            'TPRODUTO1': currentRow.CDPRODUTOINI ? '': 'T', 
            'NMPRODIN': currentRow.NMPRODUTOINI,
            'NMPRODFIN': currentRow.NMPRODUTOFIN,
            'NACCONSIDERA': currentRow.NCONSIDERASUBGRUPO,
            'NACCONSIDERA1': currentRow.NCONSIDERASUBGRUPO,
            'CDGRUPFILI': currentRow.NMGRUPFILI || 'T',
            'TODOSGRUPFILI': allBranchOfficeGrouping,
            'TODOSGRUPFILI1': allBranchOfficeGrouping,
            'NRORG': getLocalVar('NRORG'),
            'NRORG1': getLocalVar('NRORG'),
        };
        const beginGenerationDateParams = [
            'DTINIAPUPREC',
            'DTINIAPUPREC1',
            'DTINIPER',
            'DTINIPER1',
        ];
        const endGenerationDateParams = [
            'DTFINAPUPREC',
            'DTFINAPUPREC1',
            'DTFINPER',
            'DTFINPER1',
        ];
        const dateParamValue = isBirt ? null : '00:00:00';
        let todosGer;
        
        if (Util.isEmptyOrBlank(currentRow.PERIODO_GERACAO.start) || Util.isEmptyOrBlank(currentRow.PERIODO_GERACAO.end)) {
            const allDateParams = [ ...beginGenerationDateParams, ...endGenerationDateParams ];
            
            allDateParams.forEach(param => {
                parameters[param] = dateParamValue;
            });
            
            todosGer = 'T';
        } else {
            beginGenerationDateParams.forEach(param => {
               parameters[param] = currentRow.PERIODO_GERACAO.start; 
            });
            
            endGenerationDateParams.forEach(param => {
               parameters[param] = currentRow.PERIODO_GERACAO.end; 
            });
            
            todosGer = isBirt ? null : '';
        }
        
        const beginCompletionDateParams = [
            'DTCONAPUPRECINI',
            'DTCONAPUPRECINI1',
        ];
        
        const endCompletionDateParams = [
            'DTCONAPUPRECFIN',
            'DTCONAPUPRECFIN1',
        ];
        
        let beginCompletionDateValue;
        let endCompletionDateValue;
        let todosCon;
        
        if (Util.isEmptyOrBlank(currentRow.PERIODO_CONCLUSAO.start) || Util.isEmptyOrBlank(currentRow.PERIODO_CONCLUSAO.end)) {
            beginCompletionDateValue = dateParamValue;
            endCompletionDateValue = dateParamValue;
            
            todosCon = 'T';
        } else {
            beginCompletionDateValue = currentRow.PERIODO_CONCLUSAO.start;
            endCompletionDateValue = currentRow.PERIODO_CONCLUSAO.end;
            
            todosCon = isBirt ? null : '';
        }
        
        beginCompletionDateParams.forEach(param => {
           parameters[param] = beginCompletionDateValue; 
        });
        
        endCompletionDateParams.forEach(param => {
           parameters[param] = endCompletionDateValue; 
        });
        
        let todosProd;
        
        if (Util.isEmptyOrBlank(currentRow.NMPRODUTOINI) && Util.isEmptyOrBlank(currentRow.NMPRODUTOFIN)) {
            todosProd = 'T';
        } else {
            todosProd = isBirt ? null : '';
        }
        
        parameters.TODOSGER = todosGer;
        parameters.TODOSGER1 = todosGer;
        
        parameters.TODOSCON = todosCon;
        parameters.TODOSCON1 = todosCon;
        
        parameters.TODOSPROD = todosProd;
        parameters.TODOSPROD1 = todosProd;
        
        return parameters;
    };

    this.geraRelatorio = function(widget) {
        if(widget.isValid() && this.validaProduto(widget.currentRow.CDPRODUTOINI, widget.currentRow.CDPRODUTOFIN)) {
            const reportName = widget.currentRow.REPORT_NAME;
            General.openReportPopup(widget, this.abreRelatorio, [widget, reportName]);
        }
    };

    this.abreRelatorio = function(widget, reportName) {
        const parametros = self.montaParametros(widget);
        General.openReport('cmp_fornecimento_emissaoAutorizacaoFornecimento#/cmp07000_relacao_fornecedores_af_analise', reportName, parametros);
    };

    /**
     * Executa a função clearValue do campo escolhido.
     * @param {Object} widget - widget aonde o campo se encontra.
     * @param {string} nomeCampo - nome do campo a ser limpo.
     */
    this.limparCampo = function(widget, nomeCampo) {
        widget.getField(nomeCampo).clearValue();
    };

    /**
     * Filtra o campo de Unidade de acordo com a Região escolhida.
     * Caso não tenha, filtra por TODAS. 
     * @param {Object} field - campo 'Unidade' a ser filtrado.
     */
    this.filtraUnidades =  function(field) {
        const currentRowParent = field.getParent().currentRow;
        const regiao = currentRowParent.NMREGIAO;
        const grupFili = currentRowParent.CDGRUPFILI != "" && currentRowParent.CDGRUPFILI != undefined ? currentRowParent.CDGRUPFILI : 'T';
        
        const params = {
            "P_CDREGIAO": this.concatArrayForParams(regiao, 'T'),
            "P_CDGRUPFILI": grupFili
        };
        
        General.onClickDependentField(field, params);
    };
    
    this.validaProduto = function(prodInicial, prodFinal) {
        if (!General.comparaPratoProduto(prodInicial, prodFinal, getLocalVar('CDPICTPROD'))) {
            MessageService.showMessageByCode("PRODUTO_INICIAL_MENOR_IGUAL_FINAL");
            return false;
        }
        return true;
    };

    /**
     * Filtra o campo de Apuração de acordo com o Comprador escolhido.
     * Caso não tenha, filtra por TODOS.
     * @param {Object} field - campo 'Apuração' a ser filtrado.
     */
    this.filtraApuracao =  function(field) {
        const cur_row_comprador = field.getParent().currentRow.NMOPERADOR;
              
        const params = {
                "P_CDCOMPRADOR": this.concatArrayForParams(cur_row_comprador)
              };
        
        General.onClickDependentField(field, params);
    };

    /**
     * Habilita/Desabilita o checkbox 'Coincidir o período de geração de apuração' de
     * acordo com o campo 'Período de Geração'. Se o campo estiver preenchido,
     * o checkbox é habilitado. Caso contrário, ele é desmarcado e desabilitado.
     * @param {Object} - .
     */
    this.acionaCoincidePeriodoApuracao = function(args) {
        const widget = args.owner.field.getParent(),
              checkboxCoincPeriodo = widget.getField('COINC_PERIODO');
        
        if (widget.currentRow.PERIODO_GERACAO) {
            checkboxCoincPeriodo.setValue('S');
            checkboxCoincPeriodo.readOnly = false;
        } else {
            checkboxCoincPeriodo.setValue('N');
            checkboxCoincPeriodo.readOnly = true;
        }
    };
    
    /**
     * Habilita/Desabilita o campo 'Sub-Grupo'. Se o checkbox estiver marcado,
     * o campo se torna obrigatório. Caso contrário, o campo é desabilitado.
     * @param {Object} field - campo 'Sub-grupo'.
     */
    this.toggleSubGrupo = function(field) {
        General.toggleReadOnlySelect(field, field.widget.getField('NMSUBGRPROD'));
    };
};


Configuration(function(ContextRegister) {
    ContextRegister.register('Cmp07000_RelacaoFornecedoresAFAnalise', Cmp07000_RelacaoFornecedoresAFAnalise);
});